import unittest
import pysal
from pysal.core.IOHandlers.gwt import GwtIO
import tempfile
import os
import warnings


class test_GwtIO(unittest.TestCase):
    def setUp(self):
        self.test_file = test_file = pysal.examples.get_path('juvenile.gwt')
        self.obj = GwtIO(test_file, 'r')

    def test_close(self):
        f = self.obj
        f.close()
        self.failUnlessRaises(ValueError, f.read)

    def test_read(self):
        w = self.obj.read()
        self.assertEqual(168, w.n)
        self.assertEqual(16.678571428571427, w.mean_neighbors)
        w.transform = 'B'
        self.assertEqual([1.0], w[1].values())

    def test_seek(self):
        self.test_read()
        self.failUnlessRaises(StopIteration, self.obj.read)
        self.obj.seek(0)
        self.test_read()

    # Commented out by CRS, GWT 'w' mode removed until we can find a good solution for retaining distances.
    # see issue #153.
    # Added back by CRS,
    def test_write(self):
        w = self.obj.read()
        f = tempfile.NamedTemporaryFile(
            suffix='.gwt', dir=pysal.examples.get_path(''))
        fname = f.name
        f.close()
        o = pysal.open(fname, 'w')
        #copy the shapefile and ID variable names from the old gwt.
        # this is only available after the read() method has been called.
        #o.shpName = self.obj.shpName
        #o.varName = self.obj.varName
        o.write(w)
        o.close()
        wnew = pysal.open(fname, 'r').read()
        self.assertEqual(wnew.pct_nonzero, w.pct_nonzero)
        os.remove(fname)


if __name__ == '__main__':
    unittest.main()



import wx
import pysal
from transforms import WorldToViewTransform

__author__ = "Charles R Schmidt <schmidtc@gmail.com>"

POINT_RADIUS = 5
BORDER_COLOR = wx.Colour(0,0,0,255)
SELECTION_COLOR = wx.Colour(255,128,0,255)
NEIGHBORS_COLOR = wx.Colour(128,255,0,255)
BACKGROUND_COLOR = wx.Colour(0,0,0,0)

class WeightsMapFrame(wx.Frame):
    def __init__(self,parent=None,size=(600,600), style=wx.DEFAULT_FRAME_STYLE, geo=None, w=None):
        wx.Frame.__init__(self,parent,size=size,style=style)
        self.Bind
        self.SetTitle("Weights Inspector")
        if issubclass(type(geo),basestring):
            geo = pysal.open(geo,'r')
        self.geo = geo
        if issubclass(type(w),basestring):
            w = pysal.open(w,'r').read()
        self.w = w
        self.wm = WeightsMap(self,self.geo,self.w)

class WeightsMap(wx.Panel):
    """ Display a Weights Inspection Map """
    def __init__(self, parent, geo, w_obj):
        wx.Panel.__init__(self,parent,size=(600,600))
        self.status = parent.CreateStatusBar(3)
        self.status.SetStatusWidths([-1,-2,-2])
        self.status.SetStatusText('No Selection',0)
        self.Bind(wx.EVT_SIZE, self.onSize)
        self.Bind(wx.EVT_PAINT, self.onPaint)
        self.Bind(wx.EVT_MOUSE_EVENTS, self.onMouse)
        self.trns = 0
        self.background = (255,255,255,255)
        w,h = self.GetSize()
        self.buffer = wx.EmptyBitmapRGBA(w,h,alpha=self.trns)
        self.geo = geo
        if geo.type == pysal.cg.shapes.Polygon:
            self.drawfunc = self.drawpoly
            self.locator = pysal.cg.PolygonLocator(geo)
        elif geo.type == pysal.cg.shapes.Point:
            self.drawfunc = self.drawpt
            self.locator = pysal.cg.PointLocator(geo)
        else:
            raise TypeError, "Unsupported Type: %r"%(geo.type)
        self.w = w_obj
        self._ids = range(self.w.n)
        self.transform = WorldToViewTransform(geo.bbox,w,h)
        self.selection = None
    def onMouse(self, evt):
        x,y = evt.X,evt.Y
        X,Y = self.transform.pixel_to_world(x,y)
        if self.geo.type == pysal.cg.shapes.Polygon:
            rs = self.locator.contains_point((X,Y))
            if rs:
                selection = rs[0].id-1
                self.set_selection(selection)
            else:
                self.set_selection(None)
        else:
            print self.locator.nearest((X,Y))
    def onSize(self, evt):
        w,h = self.GetSize()
        self.buffer = wx.EmptyBitmapRGBA(w,h,alpha=self.trns)
        self.transform = WorldToViewTransform(self.geo.bbox,w,h)
        self.draw()
    def onPaint(self, evt):
        pdc = wx.PaintDC(self)
        pdc.Clear()
        self.draw()
    def draw(self):
        dc = wx.MemoryDC()
        dc.SelectObject(self.buffer)
        dc.SetBackground(wx.Brush(wx.Colour(*self.background)))
        dc.Clear()
        dc.SelectObject(wx.NullBitmap)
        self.draw_shps(self.buffer)
        cdc = wx.ClientDC(self)
        cdc.DrawBitmap(self.buffer,0,0)
    def drawpoly(self,gc,matrix,fill=False,ids=None):
        geo = self.geo
        pth = gc.CreatePath()
        if not ids:
            ids = xrange(len(geo))
        for i in ids:
            poly = geo.get(i)
            parts = poly.parts
            if poly.holes[0]:
                parts = parts+poly.holes
            for part in parts:
                x,y = part[0]
                pth.MoveToPoint(x,y)
                for x,y in part[1:]:
                    pth.AddLineToPoint(x,y)
                pth.CloseSubpath()
        pth.Transform(matrix)
        if fill:
            gc.FillPath(pth)
        gc.StrokePath(pth)
        return gc
    def drawpt(self,gc,matrix,fill=False,ids=None):
        r = POINT_RADIUS
        radius = r/2.0
        geo = self.geo
        for pt in geo:
            x,y = matrix.TransformPoint(*pt)
            gc.DrawEllipse(x-radius,y-radius,r,r)
        return gc
    def draw_shps(self, buff, fill_color=None, ids=None, fill_style=wx.SOLID):
        transform = self.transform
        dc = wx.MemoryDC()
        dc.SelectObject(buff)
        gc = wx.GraphicsContext.Create(dc)
        gc.SetPen( gc.CreatePen(wx.Pen(BORDER_COLOR,1)) )
        if fill_color != None:
            gc.SetBrush( gc.CreateBrush(wx.Brush(fill_color,fill_style)) )
            fill = True
        else:
            fill = False
        matrix = gc.CreateMatrix()
        matrix.Scale(1./transform.scale,1./-transform.scale)
        matrix.Translate(*transform.offset)
        self.drawfunc(gc,matrix,fill,ids)
    def set_selection(self,sel):
        if self.selection == sel:
            return
        self.selection = sel

        cdc = wx.ClientDC(self)
        if sel != None:
            w,h = self.transform.pixel_size
            buff = self.buffer.GetSubBitmap((0,0,w,h))
            id = self.w.id_order[sel]
            neighbors = map(self.w.id_order.index,self.w.neighbors[id])
            self.draw_shps(buff, NEIGHBORS_COLOR, neighbors)
            if sel in neighbors:
                self.draw_shps(buff, SELECTION_COLOR, [sel], wx.CROSSDIAG_HATCH)
            else:
                self.draw_shps(buff, SELECTION_COLOR, [sel])
            print sel,":",neighbors
            cdc.DrawBitmap(buff,0,0)
            stat0 = "Selection:%s"%id
            stat1 = "Neighbors:%s"%(','.join(map(str,self.w.neighbors[id])))
            stat2 = "Weights:%s"%(','.join(map(str,self.w.weights[id])))
            self.status.SetStatusText(stat0,0)
            self.status.SetStatusText(stat1,1)
            self.status.SetStatusText(stat2,2)
        else:
            self.status.SetStatusText('No Selection',0)
            self.status.SetStatusText('',1)
            self.status.SetStatusText('',2)
            cdc.DrawBitmap(self.buffer,0,0)

class WeightsMapApp(wx.App):
    def __init__(self, geo=None, w=None, redirect=False):
        self.geo = geo
        self.w = w
        wx.App.__init__(self, redirect)
    def OnInit(self):
        self.SetAppName("Weights Inspector")
        self.frame = WeightsMapFrame(None,size=(600,600),geo=self.geo,w=self.w)
        self.SetTopWindow(self.frame)
        self.frame.Show()
        return True


if __name__=='__main__':
    #shp = pysal.examples.get_path('sids2.shp')
    #w = pysal.examples.get_path('sids2.gal')
    #app = WeightsMapApp(shp,w)
    #app.MainLoop()

    shp = pysal.examples.get_path('columbus.shp')
    w = pysal.queen_from_shapefile(shp)
    app = WeightsMapApp(shp,w)
    app.MainLoop()



import pysal

__author__ = "Charles R Schmidt <schmidtc@gmail.com>"

class WorldToViewTransform(object):
    """
    An abstract class modeling a View window.
    Supports Panning, Zooming, Resizing.

    Is observable.

    Parameters:
    worldExtent -- Extent,List -- Extent of the world, left,lower,right,upper in world coords
    pixel_width -- int -- intial width of the view in pixels
    pixel_height -- int -- intial height of the view in pixels

    Notes:
    World coordinates are expected to increase the X and Y direction.
    Pixel coordinates are inverted in the Y direction.

    This class helps tranform world coordinates to screen coordinates.
    To transform a GraphicsMatrix,
    matrix.Scale(1.0/model.scale,-1.0/model.scale)
    matrix.Translate(*model.offset)
    
    The transforms will be applied in reverse order,
    The coordinates will first be translated to the origin (of the current view).
    The coordinates will then be scaled.

    Eg.
    >>> view = WorldToViewTransform([-180,-90,180,90],500,500)
    """
    def __init__(self,worldExtent,pixel_width,pixel_height):
        """ Intialize the view to the extent of the world """
        self.__pixel_width = float(pixel_width)
        self.__pixel_height = float(pixel_height)
        self.__world = worldExtent
        self.extent = worldExtent
        # In World Coords
    def __copy__(self):
        return WorldToViewTransform(self.extent,self.__pixel_width,self.__pixel_height)
    copy = __copy__
    def __get_offset(self):
        """ 
        Returns the offset of the top left corner of the current view in world coords.
        Move the world this many units to aling it with the view.
        """
        return self.__offset
    def __set_offset(self,value):
        """
        Set the Offset of the top left corner in world coords.
        """
        assert len(value) == 2
        self.__offset = value
    offset = property(fget=__get_offset,fset=__set_offset)
    def __get_scale(self):
        """ Returns the current scale in units/pixel """
        return self.__scale
    def __set_scale(self,value):
        """ Sets the current scale in units/pixel """
        self.__scale = value
    scale = property(fget=__get_scale,fset=__set_scale)
    def __get_extent(self):
        """Returns the extent of the current view in World Coordinates."""
        left,upper = self.pixel_to_world(0,0)
        right,lower = self.pixel_to_world(self.__pixel_width,self.__pixel_height)
        return pysal.cg.Rectangle(left,lower,right,upper)
    def __set_extent(self,value):
        """ Set the extent of the current view in World Coordinates.
            Preserve fixed scale, take the max of (sx,sy).

            Use this to zoom to a sepcific region when you know the region's 
            bbox in world coords.
        """
        left,lower,right,upper = value
        width = abs(right-left)
        height = abs(upper-lower)
        sx = width/self.__pixel_width
        sy = height/self.__pixel_height
        self.__scale = max(sx,sy)

        #The offset translate the world to the origin.
        #The X offset + world.left == 0
        #The Y offset + world.upper == 0

        # Move the offset a little, so that the center of the extent is in the center of the view.
        oleft = (left+(width/2.0)) - (self.__pixel_width*self.__scale/2.0)
        oupper = (upper-height/2.0) + (self.__pixel_height*self.__scale/2.0)

        #self.__offset = (-left,-upper) # in world coords
        self.__offset = (-oleft,-oupper) # in world coords
    extent = property(fget=__get_extent,fset=__set_extent)
    def __get_width(self):
        """ Returns the width of the current view in world coords """
        return self.__pixel_width*self.scale
    def __set_width(self, value):
        """
        Sets the width of the current view, value in pixels
        
        Eg.
        >>> view = WorldToViewTransform([0,0,100,100],500,500)
        >>> view.extent[:]
        [0.0, 0.0, 100.0, 100.0]
        >>> view.width = 250
        >>> view.extent[:]
        [0.0, 0.0, 50.0, 100.0]
        """
        if self.__pixel_width != value:
            self.__pixel_width = value
    width = property(fget=__get_width,fset=__set_width)
    def __get_height(self):
        """ Returns the height of the current view in world coords """
        return self.__pixel_height*self.scale
    def __set_height(self, value):
        """
        Sets the height of the current view, value in pixels
        
        Eg.
        >>> view = WorldToViewTransform([0,0,100,100],500,500)
        >>> view.extent[:]
        [0.0, 0.0, 100.0, 100.0]
        >>> view.height = 250
        >>> view.extent[:]
        [0.0, 50.0, 100.0, 100.0]
        """
        if self.__pixel_height != value:
            self.__pixel_height = value
    height = property(fget=__get_height,fset=__set_height)
    def __get_pixel_size(self):
        """
        Set and Return the current size of the view in pixels.
        """
        return self.__pixel_width,self.__pixel_height
    def __set_pixel_size(self,value):
        w,h = value
        if self.__pixel_width != w:
            self.__pixel_width = w
        if self.__pixel_height != h:
            self.__pixel_height = h
    pixel_size = property(fget=__get_pixel_size,fset=__set_pixel_size)

    def pan(self,dpx,dpy):
        """ 
        Pan the view by (dpx,dpy) pixel coordinates.
        
        Positive deltas move the world right and down.
        Negative deltas move the world left and up.

        Eg.
        >>> view = WorldToViewTransform([0,0,100,100],500,500)
        >>> view.pan(500,0)
        >>> view.extent[:]
        [-100.0, 0.0, 0.0, 100.0]
        >>> view.pan(-500,500)
        >>> view.extent[:]
        [0.0, 100.0, 100.0, 200.0]
        >>> view.pan(0,-500)
        >>> view.extent[:]
        [0.0, 0.0, 100.0, 100.0]
        >>> view.pan(490,490)
        >>> view.extent[:]
        [-98.0, 98.0, 2.0, 198.0]
        >>> view.pan(-490,-490)
        >>> view.extent[:]
        [0.0, 0.0, 100.0, 100.0]
        """
        ogx,ogy = self.__offset
        s = self.scale
        self.__offset = ogx+(dpx*s),ogy-(dpy*s)
    def pan_to(self,extent):
        initScale = self.scale
        self.extent = extent
        self.scale = initScale
    def pixel_to_world(self,px,py):
        """
        Returns the world coordinates of the Pixel (px,py).

        Eg.
        >>> view = WorldToViewTransform([0,0,100,100],500,500)
        >>> view.pixel_to_world(0,0)
        (0.0, 100.0)
        >>> view.pixel_to_world(500,500)
        (100.0, 0.0)
        """
        sx = self.scale
        sy = -sx
        ogx,ogy = self.__offset
        return px*sx - ogx, py*sy - ogy
    def world_to_pixel(self,x,y):
        """
        Returns the pixel of the world coordinate (x,y).

        Eg.
        >>> view = WorldToViewTransform([0,0,100,100],500,500)
        >>> view.world_to_pixel(0,0)
        (0.0, 500.0)
        >>> view.world_to_pixel(100,100)
        (500.0, -0.0)
        """
        sx = self.scale
        sy = -sx
        ogx,ogy = self.__offset
        return (x+ogx)/sx, (y+ogy)/sy

if __name__=="__main__":
    import doctest
    doctest.testmod()
    view = WorldToViewTransform([0,0,100,100],500,500)



"""
Diagnostics for two stage least squares regression estimations. 
        
"""

__author__ = "Luc Anselin luc.anselin@asu.edu, Nicholas Malizia nicholas.malizia@asu.edu "

from pysal.common import *
from scipy.stats import pearsonr

__all__ = ["t_stat", "pr2_aspatial", "pr2_spatial"]


def t_stat(reg, z_stat=False):
    """
    Calculates the t-statistics (or z-statistics) and associated p-values.
    [Greene2003]_

    Parameters
    ----------
    reg             : regression object
                      output instance from a regression model
    z_stat          : boolean
                      If True run z-stat instead of t-stat

    Returns
    -------    
    ts_result       : list of tuples
                      each tuple includes value of t statistic (or z
                      statistic) and associated p-value

    Examples
    --------

    We first need to import the needed modules. Numpy is needed to convert the
    data we read into arrays that ``spreg`` understands and ``pysal`` to
    perform all the analysis. The ``diagnostics`` module is used for the tests
    we will show here and the OLS and TSLS are required to run the models on
    which we will perform the tests.

    >>> import numpy as np
    >>> import pysal
    >>> import pysal.spreg.diagnostics as diagnostics
    >>> from pysal.spreg.ols import OLS
    >>> from twosls import TSLS

    Open data on Columbus neighborhood crime (49 areas) using pysal.open().
    This is the DBF associated with the Columbus shapefile.  Note that
    pysal.open() also reads data in CSV format; since the actual class
    requires data to be passed in as numpy arrays, the user can read their
    data in using any method.  

    >>> db = pysal.open(pysal.examples.get_path("columbus.dbf"),'r')

    Before being able to apply the diagnostics, we have to run a model and,
    for that, we need the input variables. Extract the CRIME column (crime
    rates) from the DBF file and make it the dependent variable for the
    regression. Note that PySAL requires this to be an numpy array of shape
    (n, 1) as opposed to the also common shape of (n, ) that other packages
    accept.

    >>> y = np.array(db.by_col("CRIME"))
    >>> y = np.reshape(y, (49,1))

    Extract INC (income) and HOVAL (home value) vector from the DBF to be used as
    independent variables in the regression.  Note that PySAL requires this to
    be an nxj numpy array, where j is the number of independent variables (not
    including a constant). By default this model adds a vector of ones to the
    independent variables passed in, but this can be overridden by passing
    constant=False.

    >>> X = []
    >>> X.append(db.by_col("INC"))
    >>> X.append(db.by_col("HOVAL"))
    >>> X = np.array(X).T

    Run an OLS regression. Since it is a non-spatial model, all we need is the
    dependent and the independent variable.

    >>> reg = OLS(y,X)

    Now we can perform a t-statistic on the model:

    >>> testresult = diagnostics.t_stat(reg)
    >>> print("%12.12f"%testresult[0][0], "%12.12f"%testresult[0][1], "%12.12f"%testresult[1][0], "%12.12f"%testresult[1][1], "%12.12f"%testresult[2][0], "%12.12f"%testresult[2][1])
    ('14.490373143689', '0.000000000000', '-4.780496191297', '0.000018289595', '-2.654408642718', '0.010874504910')

    We can also use the z-stat. For that, we re-build the model so we consider
    HOVAL as endogenous, instrument for it using DISCBD and carry out two
    stage least squares (TSLS) estimation.

    >>> X = []
    >>> X.append(db.by_col("INC"))
    >>> X = np.array(X).T    
    >>> yd = []
    >>> yd.append(db.by_col("HOVAL"))
    >>> yd = np.array(yd).T
    >>> q = []
    >>> q.append(db.by_col("DISCBD"))
    >>> q = np.array(q).T

    Once the variables are read as different objects, we are good to run the
    model.

    >>> reg = TSLS(y, X, yd, q)

    With the output of the TSLS regression, we can perform a z-statistic:

    >>> testresult = diagnostics.t_stat(reg, z_stat=True)
    >>> print("%12.10f"%testresult[0][0], "%12.10f"%testresult[0][1], "%12.10f"%testresult[1][0], "%12.10f"%testresult[1][1], "%12.10f"%testresult[2][0], "%12.10f"%testresult[2][1])
    ('5.8452644705', '0.0000000051', '0.3676015668', '0.7131703463', '-1.9946891308', '0.0460767956')
    """

    k = reg.k           # (scalar) number of ind. vas (includes constant)
    n = reg.n           # (scalar) number of observations
    vm = reg.vm         # (array) coefficients of variance matrix (k x k)
    betas = reg.betas   # (array) coefficients of the regressors (1 x k)
    variance = vm.diagonal()
    tStat = betas.reshape(len(betas),) / np.sqrt(variance)
    ts_result = []
    for t in tStat:
        if z_stat:
            ts_result.append((t, stats.norm.sf(abs(t)) * 2))
        else:
            ts_result.append((t, stats.t.sf(abs(t), n - k) * 2))
    return ts_result


def pr2_aspatial(tslsreg):
    """
    Calculates the pseudo r^2 for the two stage least squares regression.

    Parameters
    ----------
    tslsreg             : two stage least squares regression object
                          output instance from a two stage least squares
                          regression model

    Returns
    -------
    pr2_result          : float
                          value of the squared pearson correlation between
                          the y and tsls-predicted y vectors

    Examples
    --------

    We first need to import the needed modules. Numpy is needed to convert the
    data we read into arrays that ``spreg`` understands and ``pysal`` to
    perform all the analysis. The TSLS is required to run the model on
    which we will perform the tests.

    >>> import numpy as np
    >>> import pysal
    >>> from twosls import TSLS

    Open data on Columbus neighborhood crime (49 areas) using pysal.open().
    This is the DBF associated with the Columbus shapefile.  Note that
    pysal.open() also reads data in CSV format; since the actual class
    requires data to be passed in as numpy arrays, the user can read their
    data in using any method.  

    >>> db = pysal.open(pysal.examples.get_path("columbus.dbf"),'r')

    Before being able to apply the diagnostics, we have to run a model and,
    for that, we need the input variables. Extract the CRIME column (crime
    rates) from the DBF file and make it the dependent variable for the
    regression. Note that PySAL requires this to be an numpy array of shape
    (n, 1) as opposed to the also common shape of (n, ) that other packages
    accept.

    >>> y = np.array(db.by_col("CRIME"))
    >>> y = np.reshape(y, (49,1))

    Extract INC (income) vector from the DBF to be used as
    independent variables in the regression.  Note that PySAL requires this to
    be an nxj numpy array, where j is the number of independent variables (not
    including a constant). By default this model adds a vector of ones to the
    independent variables passed in, but this can be overridden by passing
    constant=False.

    >>> X = []
    >>> X.append(db.by_col("INC"))
    >>> X = np.array(X).T

    In this case, we consider HOVAL (home value) as an endogenous regressor,
    so we acknowledge that by reading it in a different category.

    >>> yd = []
    >>> yd.append(db.by_col("HOVAL"))
    >>> yd = np.array(yd).T

    In order to properly account for the endogeneity, we have to pass in the
    instruments. Let us consider DISCBD (distance to the CBD) is a good one:

    >>> q = []
    >>> q.append(db.by_col("DISCBD"))
    >>> q = np.array(q).T

    Now we are good to run the model. It is an easy one line task.

    >>> reg = TSLS(y, X, yd, q=q)

    In order to perform the pseudo R^2, we pass the regression object to the
    function and we are done!

    >>> result = pr2_aspatial(reg)
    >>> print("%1.6f"%result)    
    0.279361

    """

    y = tslsreg.y
    predy = tslsreg.predy
    pr = pearsonr(y, predy)[0]
    pr2_result = float(pr ** 2)
    return pr2_result


def pr2_spatial(tslsreg):
    """
    Calculates the pseudo r^2 for the spatial two stage least squares 
    regression.

    Parameters
    ----------
    stslsreg            : spatial two stage least squares regression object
                          output instance from a spatial two stage least 
                          squares regression model

    Returns
    -------    
    pr2_result          : float
                          value of the squared pearson correlation between
                          the y and stsls-predicted y vectors

    Examples
    --------

    We first need to import the needed modules. Numpy is needed to convert the
    data we read into arrays that ``spreg`` understands and ``pysal`` to
    perform all the analysis. The GM_Lag is required to run the model on
    which we will perform the tests and the ``pysal.spreg.diagnostics`` module
    contains the function with the test.

    >>> import numpy as np
    >>> import pysal
    >>> import pysal.spreg.diagnostics as D
    >>> from twosls_sp import GM_Lag

    Open data on Columbus neighborhood crime (49 areas) using pysal.open().
    This is the DBF associated with the Columbus shapefile.  Note that
    pysal.open() also reads data in CSV format; since the actual class
    requires data to be passed in as numpy arrays, the user can read their
    data in using any method.  

    >>> db = pysal.open(pysal.examples.get_path("columbus.dbf"),'r')

    Extract the HOVAL column (home value) from the DBF file and make it the
    dependent variable for the regression. Note that PySAL requires this to be
    an numpy array of shape (n, 1) as opposed to the also common shape of (n, )
    that other packages accept.

    >>> y = np.array(db.by_col("HOVAL"))
    >>> y = np.reshape(y, (49,1))

    Extract INC (income) vectors from the DBF to be used as
    independent variables in the regression.  Note that PySAL requires this to
    be an nxj numpy array, where j is the number of independent variables (not
    including a constant). By default this model adds a vector of ones to the
    independent variables passed in, but this can be overridden by passing
    constant=False.

    >>> X = np.array(db.by_col("INC"))
    >>> X = np.reshape(X, (49,1))

    In this case, we consider CRIME (crime rates) as an endogenous regressor,
    so we acknowledge that by reading it in a different category.

    >>> yd = np.array(db.by_col("CRIME"))
    >>> yd = np.reshape(yd, (49,1))

    In order to properly account for the endogeneity, we have to pass in the
    instruments. Let us consider DISCBD (distance to the CBD) is a good one:

    >>> q = np.array(db.by_col("DISCBD"))
    >>> q = np.reshape(q, (49,1))

    Since this test has a spatial component, we need to specify the spatial
    weights matrix that includes the spatial configuration of the observations
    into the error component of the model. To do that, we can open an already
    existing gal file or create a new one. In this case, we will create one
    from ``columbus.shp``.

    >>> w = pysal.rook_from_shapefile(pysal.examples.get_path("columbus.shp")) 

    Unless there is a good reason not to do it, the weights have to be
    row-standardized so every row of the matrix sums to one. Among other
    things, this allows to interpret the spatial lag of a variable as the
    average value of the neighboring observations. In PySAL, this can be
    easily performed in the following way:

    >>> w.transform = 'r'

    Now we are good to run the spatial lag model. Make sure you pass all the
    parameters correctly and, if desired, pass the names of the variables as
    well so when you print the summary (reg.summary) they are included:

    >>> reg = GM_Lag(y, X, w=w, yend=yd, q=q, w_lags=2, name_x=['inc'], name_y='hoval', name_yend=['crime'], name_q=['discbd'], name_ds='columbus')

    Once we have a regression object, we can perform the spatial version of
    the pesudo R^2. It is as simple as one line!

    >>> result = pr2_spatial(reg)
    >>> print("%1.6f"%result)
    0.299649

    """

    y = tslsreg.y
    predy_e = tslsreg.predy_e
    pr = pearsonr(y, predy_e)[0]
    pr2_result = float(pr ** 2)
    return pr2_result


def _test():
    import doctest
    doctest.testmod()


if __name__ == '__main__':
    _test()



"""
Code adapted from source from:
David Eppstein, UC Irvine, 8 Mar 2002
"""

from __future__ import generators

class PriorityDictionary(dict):

    def __init__(self):
        """
        Initialize PriorityDictionary by creating binary heap
        of pairs (value,key).  Note that changing or removing a dict entry will
        not remove the old pair from the heap until it is found by smallest() or
        until the heap is rebuilt.
        """
        self.__heap = []
        dict.__init__(self)

    def smallest(self):
        """
        Find smallest item after removing deleted items from heap.
        """
        if len(self) == 0:
            raise IndexError, "smallest of empty PriorityDictionary"
        heap = self.__heap
        while heap[0][1] not in self or self[heap[0][1]] != heap[0][0]:
            lastItem = heap.pop()
            insertionPoint = 0
            while 1:
                smallChild = 2*insertionPoint+1
                if smallChild+1 < len(heap) and \
                        heap[smallChild] > heap[smallChild+1]:
                    smallChild += 1
                if smallChild >= len(heap) or lastItem <= heap[smallChild]:
                    heap[insertionPoint] = lastItem
                    break
                heap[insertionPoint] = heap[smallChild]
                insertionPoint = smallChild
        return heap[0][1]
	
    def __iter__(self):
        """
        Create destructive sorted iterator of PriorityDictionary.
        """
        def iterfn():
            while len(self) > 0:
                x = self.smallest()
                yield x
                del self[x]
        return iterfn()
	
    def __setitem__(self,key,val):
        """
        Change value stored in dictionary and add corresponding
        pair to heap.  Rebuilds the heap if the number of deleted items grows
        too large, to avoid memory leakage.
        """
        dict.__setitem__(self,key,val)
        heap = self.__heap
        if len(heap) > 2 * len(self):
            self.__heap = [(v,k) for k,v in self.iteritems()]
            self.__heap.sort()  # builtin sort likely faster than O(n) heapify
        else:
            newPair = (val,key)
            insertionPoint = len(heap)
            heap.append(None)
            while insertionPoint > 0 and \
                    newPair < heap[(insertionPoint-1)//2]:
                heap[insertionPoint] = heap[(insertionPoint-1)//2]
                insertionPoint = (insertionPoint-1)//2
            heap[insertionPoint] = newPair
	
    def setdefault(self,key,val):
        """
        Reimplement setdefault to call our customized __setitem__.
        """
        if key not in self:
            self[key] = val
        return self[key]





import unittest
import pysal
import tempfile
import os

class test_DBF(unittest.TestCase):
    def setUp(self):
        self.test_file = test_file = pysal.examples.get_path('10740.dbf')
        self.dbObj = pysal.core.IOHandlers.pyDbfIO.DBF(test_file, 'r')

    def test_len(self):
        self.assertEqual(len(self.dbObj), 195)

    def test_tell(self):
        self.assertEqual(self.dbObj.tell(), 0)
        self.dbObj.read(1)
        self.assertEqual(self.dbObj.tell(), 1)
        self.dbObj.read(50)
        self.assertEqual(self.dbObj.tell(), 51)
        self.dbObj.read()
        self.assertEqual(self.dbObj.tell(), 195)
    
    def test_cast(self):
        self.assertEqual(self.dbObj._spec, [])
        self.dbObj.cast('FIPSSTCO', float)
        self.assertEqual(self.dbObj._spec[1], float)

    def test_seek(self):
        self.dbObj.seek(0)
        self.assertEqual(self.dbObj.tell(), 0)
        self.dbObj.seek(55)
        self.assertEqual(self.dbObj.tell(), 55)
        self.dbObj.read(1)
        self.assertEqual(self.dbObj.tell(), 56)

    def test_read(self):
        self.dbObj.seek(0)
        objs = self.dbObj.read()
        self.assertEqual(len(objs), 195)
        self.dbObj.seek(0)
        objsB = list(self.dbObj)
        self.assertEqual(len(objsB), 195)
        for rowA, rowB in zip(objs, objsB):
            self.assertEqual(rowA, rowB)

    def test_random_access(self):
        self.dbObj.seek(0)
        db0 = self.dbObj.read(1)[0]
        self.assertEqual(db0, [1, '35001', '000107', '35001000107', '1.07'])
        self.dbObj.seek(57)
        db57 = self.dbObj.read(1)[0]
        self.assertEqual(db57, [58, '35001', '001900', '35001001900', '19'])
        self.dbObj.seek(32)
        db32 = self.dbObj.read(1)[0]
        self.assertEqual(db32, [33, '35001', '000500', '35001000500', '5'])
        self.dbObj.seek(0)
        self.assertEqual(next(self.dbObj), db0)
        self.dbObj.seek(57)
        self.assertEqual(next(self.dbObj), db57)
        self.dbObj.seek(32)
        self.assertEqual(next(self.dbObj), db32)

    def test_write(self):
        f = tempfile.NamedTemporaryFile(suffix='.dbf')
        fname = f.name
        f.close()
        self.dbfcopy = fname
        self.out = pysal.core.IOHandlers.pyDbfIO.DBF(fname, 'w')
        self.dbObj.seek(0)
        self.out.header = self.dbObj.header
        self.out.field_spec = self.dbObj.field_spec
        for row in self.dbObj:
            self.out.write(row)
        self.out.close()

        orig = open(self.test_file, 'rb')
        copy = open(self.dbfcopy, 'rb')
        orig.seek(32)  # self.dbObj.header_size) #skip the header, file date has changed
        copy.seek(32)  # self.dbObj.header_size) #skip the header, file date has changed

        #PySAL writes proper DBF files with a terminator at the end, not everyone does.
        n = self.dbObj.record_size * self.dbObj.n_records  # bytes to read.
        self.assertEqual(orig.read(n), copy.read(n))
        #self.assertEquals(orig.read(1), copy.read(1)) # last byte may fail
        orig.close()
        copy.close()
        os.remove(self.dbfcopy)

    def test_writeNones(self):
        import datetime
        import time
        f = tempfile.NamedTemporaryFile(
            suffix='.dbf')
        fname = f.name
        f.close()
        db = pysal.core.IOHandlers.pyDbfIO.DBF(fname, 'w')
        db.header = ["recID", "date", "strID", "aFloat"]
        db.field_spec = [('N', 10, 0), ('D', 8, 0), ('C', 10, 0), ('N', 5, 5)]
        records = []
        for i in range(10):
            d = datetime.date(*time.localtime()[:3])
            rec = [i + 1, d, str(i + 1), (i + 1) / 2.0]
            records.append(rec)
        records.append([None, None, '', None])
        records.append(rec)
        for rec in records:
            db.write(rec)
        db.close()
        db2 = pysal.core.IOHandlers.pyDbfIO.DBF(fname, 'r')
        self.assertEqual(records, db2.read())

        os.remove(fname)

if __name__ == '__main__':
    unittest.main()



"""
A library of spatial network k-function functions.
Not to be used without permission.

Contact: 

Andrew Winslow
GeoDa Center for Geospatial Analysis
Arizona State University
Tempe, AZ
Andrew.Winslow@asu.edu
"""

import unittest
import test 

def _fxrange(start, end, incr):
    """
    A float version of the xrange() built-in function.

    _fxrange(number, number, number) -> iterator

    Arguments:
    start -- the lower end of the range (inclusive)
    end -- the upper end of the range (exclusive)
    incr -- the step size. must be positive.
    """
    i = 0
    while True:
        t = start + i*incr
        if t >= end:
            break
        yield t
        i += 1    

def _binary_search(list, q):
    """
    Returns the index in a list where an item should be found.
     
    Arguments:
    list -- a list of items
    q -- a value to be searched for
    """
    l = 0
    r = len(list)
    while l < r:
        m = (l + r)/2
        if list[m] > q:
            r = m
        else:
            l = m + 1
    return l

def kt_values(t_specs, distances, scaling_const):
    """
    Returns a dictionary of t numerics to k(t) numerics.

    kt_values(number list, number list, number) -> number to number dictionary

    Arguments:
    t_specs -- a 3-tuple of (t_min, t_max, t_delta) specifying the t-values to compute k(t) for
    distances -- a list of distances to compute k(t) from
    scaling_const -- a constant to multiple k(t) by for each t 
    """
    ks = {}
    distances.sort()
    if type(t_specs) == tuple:
        t_specs = [t for t in _fxrange(t_specs[0], t_specs[1], t_specs[2])]
        
    for t in t_specs:
        ks[t] = scaling_const*_binary_search(distances, t)
    return ks     






"""
FileIO: Module for reading and writing various file types in a Pythonic way.
This module should not be used directly, instead...
import pysal.core.FileIO as FileIO
Readers and Writers will mimic python file objects.
.seek(n) seeks to the n'th object
.read(n) reads n objects, default == all
.next() reads the next object
"""

__author__ = "Charles R Schmidt <schmidtc@gmail.com>"

__all__ = ['FileIO']
import os.path
import struct
from warnings import warn
import pysal


class FileIO_MetaCls(type):
    """
    This Meta Class is instantiated when the class is first defined.
    All subclasses of FileIO also inherit this meta class, which registers their abilities with the FileIO registry.
    Subclasses must contain FORMATS and MODES (both are type(list))
    """
    def __new__(mcs, name, bases, dict):
        cls = type.__new__(mcs, name, bases, dict)
        if name != 'FileIO' and name != 'DataTable':
            if "FORMATS" in dict and "MODES" in dict:
                #print "Registering %s with FileIO.\n\tFormats: %r\n\tModes: %r"%(name,dict['FORMATS'],dict['MODES'])
                FileIO._register(cls, dict['FORMATS'], dict['MODES'])
            else:
                raise TypeError("FileIO subclasses must have FORMATS and MODES defined")
        return cls


class FileIO(object):  # should be a type?
    """
    How this works:
    FileIO.open(\*args) == FileIO(\*args)
    When creating a new instance of FileIO the .__new__ method intercepts
    .__new__ parses the filename to determine the fileType
    next, .__registry and checked for that type.
    Each type supports one or more modes ['r','w','a',etc]
    If we support the type and mode, an instance of the appropriate handler
    is created and returned.
    All handlers must inherit from this class, and by doing so are automatically
    added to the .__registry and are forced to conform to the prescribed API.
    The metaclass takes cares of the registration by parsing the class definition.
    It doesn't make much sense to treat weights in the same way as shapefiles and dbfs,
    ....for now we'll just return an instance of W on mode='r'
    .... on mode='w', .write will expect an instance of W
    """
    __metaclass__ = FileIO_MetaCls
    __registry = {}  # {'shp':{'r':[OGRshpReader,pysalShpReader]}}

    def __new__(cls, dataPath='', mode='r', dataFormat=None):
        """
        Intercepts the instantiation of FileIO and dispatches to the correct handler
        If no suitable handler is found a python file object is returned.
        """
        if cls is FileIO:
            try:
                newCls = object.__new__(cls.__registry[cls.getType(dataPath,
                                                                   mode, dataFormat)][mode][0])
            except KeyError:
                return open(dataPath, mode)
            return newCls
        else:
            return object.__new__(cls)

    @staticmethod
    def getType(dataPath, mode, dataFormat=None):
        """Parse the dataPath and return the data type"""
        if dataFormat:
            ext = dataFormat
        else:
            ext = os.path.splitext(dataPath)[1]
            ext = ext.replace('.', '')
            ext = ext.lower()
        if ext == 'txt':
            f = open(dataPath, 'r')
            l1 = f.readline()
            l2 = f.readline()
            if ext == 'txt':
                try:
                    n, k = l1.split(',')
                    n, k = int(n), int(k)
                    fields = l2.split(',')
                    assert len(fields) == k
                    return 'geoda_txt'
                except:
                    return ext
        return ext

    @classmethod
    def _register(cls, parser, formats, modes):
        """ This method is called automatically via the MetaClass of FileIO subclasses
        This should be private, but that hides it from the MetaClass
        """
        assert cls is FileIO
        for format in formats:
            if not format in cls.__registry:
                cls.__registry[format] = {}
            for mode in modes:
                if not mode in cls.__registry[format]:
                    cls.__registry[format][mode] = []
                cls.__registry[format][mode].append(parser)
        #cls.check()

    @classmethod
    def check(cls):
        """ Prints the contents of the registry """
        print "PySAL File I/O understands the following file extensions:"
        for key, val in cls.__registry.iteritems():
            print "Ext: '.%s', Modes: %r" % (key, val.keys())

    @classmethod
    def open(cls, *args, **kwargs):
        """ Alias for FileIO() """
        return cls(*args, **kwargs)

    class _By_Row:
        def __init__(self, parent):
            self.p = parent

        def __repr__(self):
            if not self.p.ids:
                return "keys: range(0,n)"
            else:
                return "keys: " + self.p.ids.keys().__repr__()

        def __getitem__(self, key):
            if type(key) == list:
                r = []
                if self.p.ids:
                    for k in key:
                        r.append(self.p.get(self.p.ids[k]))
                else:
                    for k in key:
                        r.append(self.p.get(k))
                return r
            if self.p.ids:
                return self.p.get(self.p.ids[key])
            else:
                return self.p.get(key)
        __call__ = __getitem__

    def __init__(self, dataPath='', mode='r', dataFormat=None):
        self.dataPath = dataPath
        self.dataObj = ''
        self.mode = mode
        #pos Should ALWAYS be in the range 0,...,n
        #for custom IDs set the ids property.
        self.pos = 0
        self.__ids = None  # {'id':n}
        self.__rIds = None
        self.closed = False
        self._spec = []
        self.header = []

    def __getitem__(self, key):
        return self.by_row.__getitem__(key)

    @property
    def by_row(self):
        return self._By_Row(self)

    def __getIds(self):
        return self.__ids

    def __setIds(self, ids):
        """ Property Method for .ids
        Takes a list of ids and maps then to a 0 based index
        Need to provide a method to set ID's based on a fieldName
        preferably without reading the whole file.
        """
        if isinstance(ids, list):
            try:
                assert len(ids) == len(set(ids))
            except AssertionError:
                raise KeyError("IDs must be unique")
            # keys: ID values: i
            self.__ids = {}
            # keys: i values: ID
            self.__rIds = {}
            for i, id in enumerate(ids):
                self.__ids[id] = i
                self.__rIds[i] = id
        elif isinstance(ids, dict):
            self.__ids = ids
            self.__rIds = {}
            for id, n in ids.iteritems():
                self.__rIds[n] = id
        elif not ids:
            self.__ids = None
            self.__rIds = None
    ids = property(fget=__getIds, fset=__setIds)

    @property
    def rIds(self):
        return self.__rIds

    def __iter__(self):
        self.seek(0)
        return self

    @staticmethod
    def _complain_ifclosed(closed):
        """ from StringIO """
        if closed:
            raise ValueError("I/O operation on closed file")

    def cast(self, key, typ):
        """cast key as typ"""
        if key in self.header:
            if not self._spec:
                self._spec = [lambda x:x for k in self.header]
            if typ is None:
                self._spec[self.header.index(key)] = lambda x: x
            else:
                try:
                    assert hasattr(typ, '__call__')
                    self._spec[self.header.index(key)] = typ
                except AssertionError:
                    raise TypeError('Cast Objects must be callable')
        else:
            raise KeyError("%s" % key)

    def _cast(self, row):
        if self._spec and row:
            try:
                return [f(v) for f, v in zip(self._spec, row)]
            except ValueError:
                r = []
                for f, v in zip(self._spec, row):
                    try:
                        if not v and f != str:
                            raise ValueError
                        r.append(f(v))
                    except ValueError:
                        warn("Value '%r' could not be cast to %s, value set to pysal.MISSINGVALUE" % (v, str(f)), RuntimeWarning)
                        r.append(pysal.MISSINGVALUE)
                return r

        else:
            return row

    def next(self):
        """A FileIO object is its own iterator, see StringIO"""
        self._complain_ifclosed(self.closed)
        r = self.__read()
        if r is None:
            raise StopIteration
        return r

    def close(self):
        """ subclasses should clean themselves up and then call this method """
        if not self.closed:
            self.closed = True
            del self.dataObj, self.pos

    def get(self, n):
        """ Seeks the file to n and returns n
        If .ids is set n should be an id,
        else, n should be an offset
        """
        prevPos = self.tell()
        self.seek(n)
        obj = self.__read()
        self.seek(prevPos)
        return obj

    def seek(self, n):
        """ Seek the FileObj to the beginning of the n'th record,
            if ids are set, seeks to the beginning of the record at id, n"""
        self._complain_ifclosed(self.closed)
        self.pos = n

    def tell(self):
        """ Return id (or offset) of next object """
        self._complain_ifclosed(self.closed)
        return self.pos

    def read(self, n=-1):
        """ Read at most n objects, less if read hits EOF
        if size is negative or omitted read all objects until EOF
        returns None if EOF is reached before any objects.
        """
        self._complain_ifclosed(self.closed)
        if n < 0:
            #return list(self)
            result = []
            while 1:
                try:
                    result.append(self.__read())
                except StopIteration:
                    break
            return result
        elif n == 0:
            return None
        else:
            result = []
            for i in range(0, n):
                try:
                    result.append(self.__read())
                except StopIteration:
                    break
            return result

    def __read(self):
        """ Gets one row from the file handler, and if necessary casts it's objects """
        row = self._read()
        if row is None:
            raise StopIteration
        row = self._cast(row)
        return row

    def _read(self):
        """ Must be implemented by subclasses that support 'r'
        subclasses should increment .pos
        and redefine this doc string
        """
        self._complain_ifclosed(self.closed)
        raise NotImplementedError

    def truncate(self, size=None):
        """ Should be implemented by subclasses
        and redefine this doc string
        """
        self._complain_ifclosed(self.closed)
        raise NotImplementedError

    def write(self, obj):
        """ Must be implemented by subclasses that support 'w'
        subclasses should increment .pos
        subclasses should also check if obj is an instance of type(list)
        and redefine this doc string
        """
        self._complain_ifclosed(self.closed)
        "Write obj to dataObj"
        raise NotImplementedError

    def flush(self):
        self._complain_ifclosed(self.closed)
        raise NotImplementedError
